Method: Paperclip.options
- Defined in:
- lib/paperclip.rb
.options ⇒ Object
Provides configurability to Paperclip. The options available are:
- whiny: Will raise an error if Paperclip cannot process thumbnails of an uploaded image. Defaults to true.
- log: Logs progress to the Rails log. Uses ActiveRecord's logger, so honors log levels, etc. Defaults to true.
- command_path: Defines the path at which to find the command line programs if they are not visible to Rails the system's search path. Defaults to nil, which uses the first executable found in the user's search path.
- use_exif_orientation: Whether to inspect EXIF data to determine an image's orientation. Defaults to true.
120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 |
# File 'lib/paperclip.rb', line 120 def self. @options ||= { command_path: nil, content_type_mappings: {}, log: true, log_command: true, read_timeout: nil, swallow_stderr: true, use_exif_orientation: true, whiny: true, is_windows: Gem.win_platform?, add_validation_errors_to: :both, backend: :image_magick } end |