Class: Paperclip::Attachment

Inherits:
Object
  • Object
show all
Defined in:
lib/paperclip/attachment.rb

Overview

The Attachment class manages the files for a given attachment. It saves when the model saves, deletes when the model is destroyed, and processes the file upon assignment.

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(name, instance, options = {}) ⇒ Attachment

Creates an Attachment object. name is the name of the attachment, instance is the model object instance it’s attached to, and options is the same as the hash passed to has_attached_file.

Options include:

url - a relative URL of the attachment. This is interpolated using interpolator path - where on the filesystem to store the attachment. This is interpolated using interpolator styles - a hash of options for processing the attachment. See has_attached_file for the details only_process - style args to be run through the post-processor. This defaults to the empty list (which is

a special case that indicates all styles should be processed)

default_url - a URL for the missing image default_style - the style to use when an argument is not specified e.g. #url, #path storage - the storage mechanism. Defaults to :filesystem use_timestamp - whether to append an anti-caching timestamp to image URLs. Defaults to true whiny, whiny_thumbnails - whether to raise when thumbnailing fails use_default_time_zone - related to use_timestamp. Defaults to true hash_digest - a string representing a class that will be used to hash URLs for obfuscation hash_data - the relative URL for the hash data. This is interpolated using interpolator hash_secret - a secret passed to the hash_digest convert_options - flags passed to the convert command for processing source_file_options - flags passed to the convert command that controls how the file is read processors - classes that transform the attachment. Defaults to [:thumbnail] preserve_files - whether to keep files on the filesystem when deleting or clearing the attachment. Defaults to false filename_cleaner - An object that responds to #call(filename) that will strip unacceptable charcters from filename interpolator - the object used to interpolate filenames and URLs. Defaults to Paperclip::Interpolations url_generator - the object used to generate URLs, using the interpolator. Defaults to Paperclip::UrlGenerator escape_url - Perform URI escaping to URLs. Defaults to true return_file_attributes_on_destroy - whether attachment-related attributes should be displayed when

destroying a record. Defaults to false.


77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
# File 'lib/paperclip/attachment.rb', line 77

def initialize(name, instance, options = {})
  @name              = name.to_sym
  @name_string       = name.to_s
  @instance          = instance

  options = self.class.default_options.deep_merge(options)

  @options               = options
  @post_processing       = true
  @queued_for_delete     = []
  @queued_for_write      = {}
  @errors                = {}
  @dirty                 = false
  @interpolator          = options[:interpolator]
  @url_generator         = options[:url_generator].new(self)
  @source_file_options   = options[:source_file_options]
  @whiny                 = options[:whiny]

  initialize_storage
end

Instance Attribute Details

#convert_optionsObject (readonly)

Returns the value of attribute convert_options.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def convert_options
  @convert_options
end

#default_styleObject (readonly)

Returns the value of attribute default_style.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def default_style
  @default_style
end

#instanceObject (readonly)

Returns the value of attribute instance.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def instance
  @instance
end

#interpolatorObject (readonly)

Returns the value of attribute interpolator.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def interpolator
  @interpolator
end

#nameObject (readonly)

Returns the value of attribute name.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def name
  @name
end

#optionsObject (readonly)

Returns the value of attribute options.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def options
  @options
end

#post_processingObject

Returns the value of attribute post_processing.



45
46
47
# File 'lib/paperclip/attachment.rb', line 45

def post_processing
  @post_processing
end

#queued_for_deleteObject (readonly)

Returns the value of attribute queued_for_delete.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def queued_for_delete
  @queued_for_delete
end

#queued_for_writeObject (readonly)

Returns the value of attribute queued_for_write.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def queued_for_write
  @queued_for_write
end

#source_file_optionsObject (readonly)

Returns the value of attribute source_file_options.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def source_file_options
  @source_file_options
end

#whinyObject (readonly)

Returns the value of attribute whiny.



43
44
45
# File 'lib/paperclip/attachment.rb', line 43

def whiny
  @whiny
end

Class Method Details

.default_optionsObject



13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
# File 'lib/paperclip/attachment.rb', line 13

def self.default_options
  @default_options ||= {
    convert_options:                   {},
    default_style:                     :original,
    default_url:                       "/:attachment/:style/missing.png",
    escape_url:                        true,
    restricted_characters:             /[&$+,\/:;=?@<>\[\]\{\}\|\\\^~%# ]/,
    filename_cleaner:                  nil,
    hash_data:                         ":class/:attachment/:id/:style/:updated_at",
    hash_digest:                       "SHA1",
    interpolator:                      Paperclip::Interpolations,
    only_process:                      [],
    path:                              ":rails_root/public:url",
    preserve_files:                    false,
    processors:                        [:thumbnail],
    source_file_options:               {},
    storage:                           :filesystem,
    styles:                            {},
    url:                               "/system/:class/:attachment/:id_partition/:style/:filename",
    url_generator:                     Paperclip::UrlGenerator,
    use_default_time_zone:             true,
    use_timestamp:                     true,
    whiny:                             Paperclip.options[:whiny] || Paperclip.options[:whiny_thumbnails],
    validate_media_type:               true,
    adapter_options:                   { hash_digest: Digest::MD5 },
    check_validity_before_processing:  true,
    return_file_attributes_on_destroy: false
  }
end

Instance Method Details

#as_json(options = nil) ⇒ Object



192
193
194
# File 'lib/paperclip/attachment.rb', line 192

def as_json(options = nil)
  to_s((options && options[:style]) || default_style)
end

#assign(uploaded_file) ⇒ Object

What gets called when you call instance.attachment = File. It clears errors, assigns attributes, and processes the file. It also queues up the previous file for deletion, to be flushed away on #save of its host. In addition to form uploads, you can also assign another Paperclip attachment:

new_user.avatar = old_user.avatar


104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
# File 'lib/paperclip/attachment.rb', line 104

def assign(uploaded_file)
  @file = Paperclip.io_adapters.for(uploaded_file,
                                    @options[:adapter_options])
  ensure_required_accessors!
  ensure_required_validations!

  if @file.assignment?
    clear(*only_process)

    if @file.nil?
      nil
    else
      assign_attributes
      post_process_file
      reset_file_if_original_reprocessed
    end
  end
end

#blank?Boolean

Returns:

  • (Boolean)


368
369
370
# File 'lib/paperclip/attachment.rb', line 368

def blank?
  not present?
end

#clear(*styles_to_clear) ⇒ Object

Clears out the attachment. Has the same effect as previously assigning nil to the attachment. Does NOT save. If you wish to clear AND save, use #destroy.



253
254
255
256
257
258
259
260
261
# File 'lib/paperclip/attachment.rb', line 253

def clear(*styles_to_clear)
  if styles_to_clear.any?
    queue_some_for_delete(*styles_to_clear)
  else
    queue_all_for_delete
    @queued_for_write  = {}
    @errors            = {}
  end
end

#content_typeObject

Returns the content_type of the file as originally assigned, and lives in the <attachment>_content_type attribute of the model.



296
297
298
# File 'lib/paperclip/attachment.rb', line 296

def content_type
  instance_read(:content_type)
end

#created_atObject

Returns the creation time of the file as originally assigned, and lives in the <attachment>_created_at attribute of the model.



302
303
304
305
306
307
# File 'lib/paperclip/attachment.rb', line 302

def created_at
  if able_to_store_created_at?
    time = instance_read(:created_at)
    time && time.to_f.to_i
  end
end

#default_optionsObject



154
155
156
157
158
159
# File 'lib/paperclip/attachment.rb', line 154

def default_options
  {
    timestamp: @options[:use_timestamp],
    escape: @options[:escape_url]
  }
end

#destroyObject

Destroys the attachment. Has the same effect as previously assigning nil to the attachment *and saving*. This is permanent. If you wish to wipe out the existing attachment but not save, use #clear.



266
267
268
269
# File 'lib/paperclip/attachment.rb', line 266

def destroy
  clear
  save
end

#dirty?Boolean

Returns true if there are changes that need to be saved.

Returns:

  • (Boolean)


235
236
237
# File 'lib/paperclip/attachment.rb', line 235

def dirty?
  @dirty
end

#errorsObject

Returns an array containing the errors on this attachment.



230
231
232
# File 'lib/paperclip/attachment.rb', line 230

def errors
  @errors
end

#expiring_url(_time = 3600, style_name = default_style) ⇒ Object

Alias to url that allows using the expiring_url method provided by the cloud storage implementations, but keep using filesystem storage for development and testing.



164
165
166
# File 'lib/paperclip/attachment.rb', line 164

def expiring_url(_time = 3600, style_name = default_style)
  url(style_name)
end

#file?Boolean Also known as: present?

Returns true if a file has been assigned.

Returns:

  • (Boolean)


362
363
364
# File 'lib/paperclip/attachment.rb', line 362

def file?
  original_filename.present?
end

#fingerprintObject

Returns the fingerprint of the file, if one’s defined. The fingerprint is stored in the <attachment>_fingerprint attribute of the model.



290
291
292
# File 'lib/paperclip/attachment.rb', line 290

def fingerprint
  instance_read(:fingerprint)
end

#hash_key(style_name = default_style) ⇒ Object

Returns a unique hash suitable for obfuscating the URL of an otherwise publicly viewable attachment.

Raises:

  • (ArgumentError)


324
325
326
327
328
329
330
# File 'lib/paperclip/attachment.rb', line 324

def hash_key(style_name = default_style)
  raise ArgumentError, "Unable to generate hash without :hash_secret" unless @options[:hash_secret]

  require "openssl" unless defined?(OpenSSL)
  data = interpolate(@options[:hash_data], style_name)
  OpenSSL::HMAC.hexdigest(OpenSSL::Digest.const_get(@options[:hash_digest]).new, @options[:hash_secret], data)
end

#instance_read(attr) ⇒ Object

Reads the attachment-specific attribute on the instance. See instance_write for more details.



388
389
390
391
# File 'lib/paperclip/attachment.rb', line 388

def instance_read(attr)
  getter = :"#{@name_string}_#{attr}"
  instance.send(getter) if instance.respond_to?(getter)
end

#instance_respond_to?(attr) ⇒ Boolean

Determines whether the instance responds to this attribute. Used to prevent calculations on fields we won’t even store.

Returns:

  • (Boolean)


374
375
376
# File 'lib/paperclip/attachment.rb', line 374

def instance_respond_to?(attr)
  instance.respond_to?(:"#{name}_#{attr}")
end

#instance_write(attr, value) ⇒ Object

Writes the attachment-specific attribute on the instance. For example, instance_write(:file_name, “me.jpg”) will write “me.jpg” to the instance’s “avatar_file_name” field (assuming the attachment is called avatar).



381
382
383
384
# File 'lib/paperclip/attachment.rb', line 381

def instance_write(attr, value)
  setter = :"#{@name_string}_#{attr}="
  instance.send(setter, value) if instance.respond_to?(setter)
end

#only_processObject



213
214
215
216
217
# File 'lib/paperclip/attachment.rb', line 213

def only_process
  only_process = @options[:only_process].dup
  only_process = only_process.call(self) if only_process.respond_to?(:call)
  only_process.map(&:to_sym)
end

#original_filenameObject

Returns the name of the file as originally assigned, and lives in the <attachment>_file_name attribute of the model.



278
279
280
# File 'lib/paperclip/attachment.rb', line 278

def original_filename
  instance_read(:file_name)
end

#path(style_name = default_style) ⇒ Object

Returns the path of the attachment as defined by the :path option. If the file is stored in the filesystem the path refers to the path of the file on disk. If the file is stored in S3, the path is the “key” part of the URL, and the :bucket option refers to the S3 bucket.



172
173
174
175
# File 'lib/paperclip/attachment.rb', line 172

def path(style_name = default_style)
  path = original_filename.nil? ? nil : interpolate(path_option, style_name)
  path.respond_to?(:unescape) ? path.unescape : path
end

#processorsObject



219
220
221
222
223
224
225
226
227
# File 'lib/paperclip/attachment.rb', line 219

def processors
  processing_option = @options[:processors]

  if processing_option.respond_to?(:call)
    processing_option.call(instance)
  else
    processing_option
  end
end

#reprocess!(*style_args) ⇒ Object

This method really shouldn’t be called that often. Its expected use is in the paperclip:refresh rake task and that’s it. It will regenerate all thumbnails forcefully, by reobtaining the original file and going through the post-process again. NOTE: Calling reprocess WILL NOT delete existing files. This is due to inconsistencies in timing of S3 commands. It’s possible that calling #reprocess! will lose data if the files are not kept.



339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
# File 'lib/paperclip/attachment.rb', line 339

def reprocess!(*style_args)
  saved_flags = @options.slice(
    :only_process,
    :preserve_files,
    :check_validity_before_processing
  )
  @options[:only_process] = style_args
  @options[:preserve_files] = true
  @options[:check_validity_before_processing] = false

  begin
    assign(self)
    save
    instance.save
  rescue Errno::EACCES => e
    warn "#{e} - skipping file."
    false
  ensure
    @options.merge!(saved_flags)
  end
end

#saveObject

Saves the file, if there are no errors. If there are, it flushes them to the instance’s errors and returns false, cancelling the save.



241
242
243
244
245
246
247
248
# File 'lib/paperclip/attachment.rb', line 241

def save
  flush_deletes unless @options[:keep_old_files]
  process = only_process
  @queued_for_write.except!(:original) if process.any? && !process.include?(:original)
  flush_writes
  @dirty = false
  true
end

#sizeObject

Returns the size of the file as originally assigned, and lives in the <attachment>_file_size attribute of the model.



284
285
286
# File 'lib/paperclip/attachment.rb', line 284

def size
  instance_read(:file_size) || (@queued_for_write[:original] && @queued_for_write[:original].size)
end

#staged?Boolean

:nodoc:

Returns:

  • (Boolean)


183
184
185
# File 'lib/paperclip/attachment.rb', line 183

def staged?
  !@queued_for_write.empty?
end

#staged_path(style_name = default_style) ⇒ Object

:nodoc:



178
179
180
# File 'lib/paperclip/attachment.rb', line 178

def staged_path(style_name = default_style)
  @queued_for_write[style_name].path if staged?
end

#stylesObject



200
201
202
203
204
205
206
207
208
209
210
211
# File 'lib/paperclip/attachment.rb', line 200

def styles
  if @options[:styles].respond_to?(:call) || @normalized_styles.nil?
    styles = @options[:styles]
    styles = styles.call(self) if styles.respond_to?(:call)

    @normalized_styles = styles.dup
    styles.each_pair do |name, options|
      @normalized_styles[name.to_sym] = Paperclip::Style.new(name.to_sym, options.dup, self)
    end
  end
  @normalized_styles
end

#time_zoneObject

The time zone to use for timestamp interpolation. Using the default time zone ensures that results are consistent across all threads.



318
319
320
# File 'lib/paperclip/attachment.rb', line 318

def time_zone
  @options[:use_default_time_zone] ? Time.zone_default : Time.zone
end

#to_s(style_name = default_style) ⇒ Object

Alias to url



188
189
190
# File 'lib/paperclip/attachment.rb', line 188

def to_s(style_name = default_style)
  url(style_name)
end

#updated_atObject

Returns the last modified time of the file as originally assigned, and lives in the <attachment>_updated_at attribute of the model.



311
312
313
314
# File 'lib/paperclip/attachment.rb', line 311

def updated_at
  time = instance_read(:updated_at)
  time && time.to_f.to_i
end

#uploaded_fileObject

Returns the uploaded file if present.



272
273
274
# File 'lib/paperclip/attachment.rb', line 272

def uploaded_file
  instance_read(:uploaded_file)
end

#url(style_name = default_style, options = {}) ⇒ Object

Returns the public URL of the attachment with a given style. This does not necessarily need to point to a file that your Web server can access and can instead point to an action in your app, for example for fine grained security; this has a serious performance tradeoff.

Options:

timestamp - Add a timestamp to the end of the URL. Default: true. escape - Perform URI escaping to the URL. Default: true.

Global controls (set on has_attached_file):

interpolator - The object that fills in a URL pattern’s variables. default_url - The image to show when the attachment has no image. url - The URL for a saved image. url_generator - The object that generates a URL. Default: Paperclip::UrlGenerator.

As mentioned just above, the object that generates this URL can be passed in, for finer control. This object must respond to two methods:

#new(Paperclip::Attachment, options_hash) #for(style_name, options_hash)



146
147
148
149
150
151
152
# File 'lib/paperclip/attachment.rb', line 146

def url(style_name = default_style, options = {})
  if options == true || options == false # Backwards compatibility.
    @url_generator.for(style_name, default_options.merge(timestamp: options))
  else
    @url_generator.for(style_name, default_options.merge(options))
  end
end