Exception: Paperclip::Errors::NotIdentifiedByBackendError

Inherits:
Paperclip::Error
  • Object
show all
Defined in:
lib/paperclip/errors.rb

Overview

Will be thrown when the backend cannot determine the uploaded file’s metadata, usually this would mean the file is not an image.